name	"SPAM_D_HETCOR_H";
title	"2D SPAM-D-HETCOR with variable CP ampl. and TPPM decoupling";

!  COMPILED WITH OPTIMIZATION ON
!  For Infinity only
!  Hypercomplex acquisition

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Version 1.1                                                                !
!   LastUpdated: 2005/05/13                                                    !
!   comments:    Jerzy W. Wiench        or  Marek Pruski                       !
!   email:       jwiench@iastate.edu        mpruski@iastate.edu                !
!   address:     Ames Laboratory, Iowa State University, 229 Spedding Hall,    !
!                Ames IA 50011-3020, USA                                       !
!	email to:   jwiench@iastate.edu                                            !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   References:                                                                !
!   MQ-J-HETCOR     J.W. Wiench, M. Pruski, Solid State NMR, 26 (2004) 51.     !
!   SPAM-D-HETCOR and SPAM-J-HETCOR   J.W.Wiench, G.Tricot, L.Delevoye,        !
!           J.Trebosc, J.Frye, L.Montagne, J.P.Amoureux, M.Pruski, Phys. Chem. !
!           Chem. Phys., (2005) accepted.                                      !
!   SPAM   	Z. Gan, H.T. Kwak, J. Magn. Reson. 168 (2004) 346.                 !
!   MQMAS   L. Frydman, J.S. Harwood, J. Am. Chem. Soc. 117 (1995) 5367.       !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   SPAM-D-HETCOR SEQUENCE:                                                    !
!   ch1:  pw1-t1-pw2-.3u-ZF_pulse-Rt1-  CP  -decouple or blank amp.            !
!   ch2:                              rampCP-Acquire                           !
!   ch3:                                    -blank amp.                        !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Inum and Pnum must be select properly.                                     !
!   For example:                                                               !
!   set Pnum=3 and Inum=2.5 for 3Q-CP-HETCOR experiment for 5/2 spins          !
!   (it set MQratio=19./12., PHI1=SPAM_3Q_e, and hypercomplex phase=30.)       !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Hyper_flag turns on (1) or off (0) hypercomplex data acquisition.          !
!   Is always sets 1 when al2>1 and you are about to collect a spectrum.       !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   RAMP_flag selects: no ramp (0), ramp on H (1) or X (2) nuclei.             !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Hdec_flag selects decoupling mode during acquisiton:                       !
!   no decoupling i.e. blanking amplifier (0), CW dec. (1) or TPPM dec. (2).   !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Uncoment line 207 if you must use different scMQ_ZF and scH.               !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!

	NMRchnls	RF: ch1 ch2 ch3; NMRacq;

! ------------------------------------------
! Define variables in .data section
! ------------------------------------------

.data
	.time			dec_time			= 2.ms;
	.float			MQratio				= 1.583333;
	.long	extern	Hdec_flag  			= 1;

!----------------------------------------------------------------!
! phases for 3/2 spin nuclei and 3Q SPAM experiment.             !
! Pathway: 0 => -3 => 1,0,-1 => -1                               !
!----------------------------------------------------------------!
	.phase	list SPAM_3Q_ae[]	=
				 60, 60, 30, 30,   0,  0, 90, 90,
				180,180,150,150, 120,120,210,210,
				300,300,270,270, 240,240,330,330;
!----------------------------------------------------------------!
! phases for 5/2 and greater spin nuclei and 3Q SPAM experiment. !
! Pathway: 0 => +3 => 1,0,-1 => -1                               !
!----------------------------------------------------------------!
	.phase  list SPAM_3Q_e[]	=
				  0,  0, 30, 30,   60, 60, 90, 90,
				120,120,150,150,  180,180,210,210,
	 			240,240,270,270,  300,300,330,330;
!----------------------------------------------------------------!
! phases for 5/2 spin nuclei and 5Q SPAM experiment.             !
! Pathway: 0 => -5 => 1,0,-1 => -1                               !
!----------------------------------------------------------------!
	.phase  list SPAM_5Q_ae[]	=
				 36, 36, 18, 18,    0,  0, 54, 54,
				108,108, 90, 90,   72, 72,126,126,
				180,144,198,162,  144,144,198,198,
				252,252,234,234,  216,216,270,270,
				324,324,306,306,  288,288,342,342;
!----------------------------------------------------------------!
! phases for 7/2 and greater spin nuclei and 5Q SPAM experiment. !
! Pathway: 0 => +5 => 1,0,-1 => -1                               !
!----------------------------------------------------------------!
	.phase  list SPAM_5Q_e[]	=
				  0,  0, 18, 18,   36, 36, 54, 54,
				 72, 72, 90, 90,  108,108,126,126,
				144,144,162,162,  180,180,198,198,
				216,216,234,234,  252,252,270,270,
				288,288,306,306,  324,324,342,342;

!----------------------------------------------------!
! Other phases are the same for both 3Q and 5Q SPAM  !
!----------------------------------------------------!
	.phase		 SPAM_0			=   0;
	.phase		 SPAM_2			= 180;
	.phase  list Hmix[]			=  90,270,  0,180,270, 90,180,  0;		!Hmix&decouple phase list
	.phase	list Xmix[]			= 270,270,180,180, 90, 90,  0,  0;		!X phase list
	.phase		 Xdec			=   0;
	.phase  PHI1,PHI2,PHI3;
	.long	extern list abph[]	= 0,2,3,1, 2,0,1,3;			!receiver cycling

include "../includes/STANDARD_PARAMS";
include "../includes/2D.inc";
include "../includes/2Dutil2.inc";
include "../includes/CP.inc";
include "../includes/TPPM.inc";
include "../includes/NA_ERRORS.inc";
include "../includes/MQMASimp.inc";

define NOTREADY_ERR 0x152
define NOTREADY_ERROR_CODE USER_ERROR_BASE + NOTREADY_ERR
comment "ERROR "NOTREADY_ERROR_CODE "Pulse phases are not ready for this combination of Pnum and Inum.";

!define T1EVOL_ERR 0x152
!define T1EVOL_ERROR_CODE USER_ERROR_BASE + T1EVOL_ERR
!comment "ERROR "T1EVOL_ERROR_CODE "t1_evol must be greater than pw180X+0.3u";
!
!-------------------------------------------
! update Spinsight with calculations. 
!-------------------------------------------

USE_TPPM_PARAMS;
USE_RAMP_PARAMS;

.update "rb=1.30*sw";
!.update "aqtm=((dw*al)/(Hyper_flag+1))";
.update "aqtm2=(dw2*al2)+t1_evol";
.update "t1_out=(t1_evol*(31./12.))";
.update "extm=(pw1+pw2+ZF_pulse+t1_out+((dw2*al2)/2.)+ct+rd+ad+aqtm+pd)";
.update "txduty1=((2.*ct)+pw1+pw2+ZF_pulse+(aqtm*((Hdec_flag+1)/2)))/extm";
.update "time1d=((na+dp)*extm*(Hyper_flag+1))/60.0";
.update "time2d=(al2*time1d)/60.";

!------------------------------------------
!  Execute Once at Start of Experiment
!------------------------------------------

.program

dpc = dp;
dec_time = rd + ad + aqtm;
CALC_HALFAL;
CALC_RAMP;

! calculate TPPM cycles and set TPPM phase excursion
CALC_TPPM_DEC(dec_time,p2);

! check if NA is correct
if		(Pnum==3)	{ if (na%24>0) {error(NA024_ERR);} }
else if	(Pnum==5)	{ if (na%40>0) {error(NA024_ERR);} }
else				{ error(NOTREADY_ERR); }
if		(Inum==1.5 && Pnum==3)	{ MQratio = 7./9.;   }
else if	(Inum==2.5 && Pnum==3)	{ MQratio = 19./12.; }
else if	(Inum==2.5 && Pnum==5)	{ MQratio = 25./12.; }
else if	(Inum==3.5 && Pnum==3)	{ MQratio = 101./45.;}
else if	(Inum==3.5 && Pnum==5)	{ MQratio = 11./9.;  }
else if	(Inum==3.5 && Pnum==7)	{ MQratio = 161./45.;}
else if	(Inum==4.5 && Pnum==3)	{ MQratio = 91./36.; }
else if	(Inum==4.5 && Pnum==5)	{ MQratio = 95./36.; }
else if	(Inum==4.5 && Pnum==7)	{ MQratio = 7./18.;  }
else if	(Inum==4.5 && Pnum==9)	{ MQratio = 31./6.;  }
else							{ error(SPIN_ERR);   }

extm=(5.5us+pw1+pw2+ZF_pulse+(t1_evol*(MQratio+1.))+ct+aqtm+pd);
txduty1=(pw1+pw2+ZF_pulse+(2.*ct))/extm;

if (Hdec_flag > 0) {
	txduty1=txduty1+aqtm/extm;
}
!t1_out=(t1_evol-pw180X)/2.;
t1_out=t1_evol;
if ( t1_out<.3u) {error(T1EVOL_ERR);}

if (txduty1 > 0.4) {error(TXDUTY_ERR);}			!Duty factor too large

!adjust pd for scan delay
pd=pd-10m;

! -------------------------------------------
! actual pulse prog. runtime loop
! -------------------------------------------

.start

loop2D {
	t1_out_1   = t1_evol + dw2*index2;
	t1_out_2   = MQratio*(t1_evol + dw2*index2);
	abph	   = abph.start;
	Hmix	   = Hmix.start;
	Xmix	   = Xmix.start;
	SPAM_3Q_e  = SPAM_3Q_e.start;
	SPAM_3Q_ae = SPAM_3Q_ae.start;
	SPAM_5Q_e  = SPAM_5Q_e.start;
	SPAM_5Q_ae = SPAM_5Q_ae.start;

	loop1D {
		aqph=@abph++;
		if		(Pnum==3 && Inum>2.1)	{ PHI1 = @SPAM_3Q_e++;  PHI3 = SPAM_0;}
		else if	(Pnum==3 && Inum==1.5)	{ PHI1 = @SPAM_3Q_ae++; PHI3 = SPAM_2;}
		else if	(Pnum==5 && Inum==2.5)	{ PHI1 = @SPAM_5Q_ae++; PHI3 = SPAM_2;}
		else if	(Pnum==5 && Inum>3.1)	{ PHI1 = @SPAM_5Q_e++;  PHI3 = SPAM_0;}
		for (Hyper = 0, Hyper < (Hyper_flag + 1), Hyper++) {
			if (Hyper==0)	{ phi_mix = 0.; }
			else			{ phi_mix = (90./Pnum); }
			call mainseq;
			if (Hyper < Hyper_flag) { out   time(10m); }	!pd, recycle delay included in mainseq
		}
		scan  time(10m);               !end of seq. bookkeeping
		Xmix++;
		Hmix++;

	}
}
.end

mainseq:

	INIT_RAMP;			!initialize ramp

	out  time(3u)		ch1: SC(scXcp)			ch2: SC(scH);
	out	 time(1u)		ch1: AP(aX,Xdec)		ch2: AP(aMQ,phi_mix+PHI1);	!preset phase & ampl.

 	out  pw1									ch2: TG;
 	out  time(t1_out_1)							ch2: P(SPAM_0);
	out  pw2									ch2: TG;
	out  time(.3u)								ch2: AP(aH,PHI3);
	!out  time(1.4u)								ch2: SC(scMQ_ZF);		!uncoment if diffrent than scH
	out  ZF_pulse								ch2: TG;
	out  time(t1_out_2)	ch1: AP(aXcp,@Xmix)		ch2: AP(aHcp,@Hmix);

	RUN_RAMP_CP(@Xmix,@Hmix);

Async;
!--------------------------- X channel ----------------------------!
ch++ ch1;
	out	  rd					ch1: TB | P(p);						!blank X amp.
	out   ad					ch1: RE | TB;
	outAQ dw					ch1: RE | TB , halfal;

	out  pd; ! secure timing for resync durind recycling delay, recycling delay
Sync;
ch--;
 
!------------------------ proton channel --------------------------!
ch++ ch2;
	out   time(.3u)				ch2: A(aHdec);                          
	if (Hdec_flag == 1) {
		out   time(dec_time)	ch2:TG;
	}
	else if (Hdec_flag == 2) {
		RUN_TPPM_DEC(ch2,cyclesPM);
	}
	else {
		out   time(dec_time)	ch2: TB;
	}
	out   time(0.1u)			ch2: TB;
waitS;
ch--;

!--------------------------- Y channel ----------------------------!
ch++ ch3;
	out   time(dec_time)		ch3: TB;
	out   time(0.1u)			ch3: TB;
waitS;
ch--;

return;
