name  "SPAM_J_HETCOR_H";
title "MQ-INEPT sequence - regular, refocused, and INEPT+, non selective PT, H decoupling included";

!  COMPILED WITH OPTIMIZATION ON
!  For Infinity only
!  Hypercomplex acquisition

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   Version 1.5                                                                !
!	LastUpdate:	2005/03/29                                                     !
!   comments:    Jerzy W. Wiench        or  Marek Pruski         or J. Trebosc !
!   email:       jwiench@iastate.edu        mpruski@iastate.edu                !
!   address:     Ames Laboratory, Iowa State University, 229 Spedding Hall,    !
!                Ames IA 50011-3020, USA                                       !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!   References:                                                                !
!   MQ-J-HETCOR     J.W. Wiench, M. Pruski, Solid State NMR, 26 (2004) 51.     !
!   SPAM-D-HETCOR and SPAM-J-HETCOR   J.W.Wiench, G.Tricot, L.Delevoye,        !
!           J.Trebosc, J.Frye, L.Montagne, J.P.Amoureux, M.Pruski, Phys. Chem. !
!           Chem. Phys., (2005) accepted.                                      !
!   SPAM   	Z. Gan, H.T. Kwak, J. Magn. Reson. 168 (2004) 346.                 !
!   MQMAS   L. Frydman, J.S. Harwood, J. Am. Chem. Soc. 117 (1995) 5367.       !
!   INEPT	G.A. Morris, R. Freeman, J. Am. Chem. Soc. 101 (1979) 760-762.     !
!   INEPTR  G.A. Morris, J. Am. Chem. Soc. 102 (1980) 428-429.                 !
!           D.P. Burum, R.R. Ernst, J. Magn. Reson. 39 (1980) 163-168.         !
!   INEPT+  O.W. Sorensen, R.R. Ernst, J. Magn. Reson. 51 (1983) 477-489.      !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! SPAM_flag switch between MQ (0) and SPAM (1) version                         !
!                                                                              !
! MQ-J-HETCOR SEQUENCE:                                                        !
! ch1:  pw1-t1-pw2-Rt1-INEPT-decouple or blank amp.                            !
! ch2:                 INEPT-Acquire                                           !
! ch3:                      -blank amp.                                        !
!                                                                              !
! SPAM-J-HETCOR SEQUENCE:                                                      !
! ch1:  pw1-t1-pw2-.3u-ZF_pulse-Rt1-INEPT-decouple or blank amp.               !
! ch2:                              INEPT-Acquire                              !
! ch3:                                   -blank amp.                           !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Inum and Pnum must be select properly.                                       !
! For example:                                                                 !
! set Pnum=3 and Inum=2.5 for 3Q-CP-HETCOR experiment for 5/2 spins            !
! (it set MQratio=19./12., PHI1=SPAM_3Q_e, and hypercomplex phase=30.)         !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Hyper_flag turns on (1) or off (0) hypercomplex data acquisition.            !
! Is always sets 1 when al2>1 and you are about to collect a spectrum.         !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Hdec_flag selects decoupling mode during acquisiton:                         !
! no decoupling i.e. blanking amplifier (0), CW dec. (1) or TPPM dec. (2).     !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! DEC_flag information : This flag has bit wise meaning :                      !
! 1st bit is related to tau1 dec                                               !
! 2nd bit is related to tau2 dec                                               !
! 3rd bit is related to acq dec                                                !
! examples : 0 -> no dec                                                       !
!            1 -> tau1 dec only                                                !
!            2 -> tau2 dec only                                                !
!            4 -> acq dec only                                                 !
!            3 -> tau1+tau2 dec                                                !
!            5 -> tau1+acq dec                                                 !
!            6 -> tau2+acq dec                                                 !
!            7 -> tau1+tau2+acq dec                                            !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!

	NMRchnls	RF: ch1 ch2 ch3; NMRacq;

! ------------------------------------------
! .data section is for compiler definitions
! ------------------------------------------

.data

	.time	extern			decT			= 0.01;
	.time	extern			decT2			=0.01;
	.time	extern			extm1			=0.01;

	!.time	extern	autofix	Tr "rotor prd"	= 200us;
	.time	extern			tau1			= .1ms;
	.time	extern			tau2			= .1ms;
	.time			taTimes[]
							StartX			= 1.us,
							TAUPW1			= 1.us,
							TAUPW2			= 1.us,
							TAU3H			= 1.us;
	.time			autofix	ttTimes[]
							TAU1X			= 1.us,
							TAU2X			= 1.us,
							TAU1Y			= 1.us,
							TAU2Y			= 1.us,
							TAU1H			= 1.us,
							TAU2H			= 1.us,
							TAU4H			= 1.us,
							TAU90H			= 1.us,
							TAU180H			= 1.us;

	.time			autofix	leftOver1,leftOver2,leftOver4;
	.time			deltaXY,deltaXH;

	.time	delays,pulses;

	.long	extern			INEPT_flag		= 0;	!0=regular, 1=refocused, 2=puls, 3=DEPT
	.long	extern			DEC_flag		= 0;	!0=off, 1=tau, 2=acq, 3=both
	.long	extern			SPAM_flag		= 0;	! if 1 use SPAM if 0 3QMAS
	.long	extern			NULL_flag		= 1;	!pw90Y is on(1) or off(0)
	.float					MQratio			= 0.5;

	.long	extern			DEPT			= 1;	! 1=45, 2=90, 3=135
	.long	cyclesPM1,cyclesPM2,cyclesPM4,cyclesPMaq;	! 1=45, 2=90, 3=135
	.long	extern			Dmode_flag		= 0;            !dcpl technique: CW, TPPM

	.ampl			autofix	a_tmp			= 0.0;

!++++++++++++++++++++++++++++++++++++
!       Pulse cycles
!++++++++++++++++++++++++++++++++++++

!------------------------------------
! phases for 5/2 spin nuclei and SPAM experiment. Pathway: 0 => +3 => 1,0,-1 => -1
!------------------------------------
	.phase	list SPAM_e[]	=
				  0,  0,  0,  0,   30, 30, 30, 30,   60, 60, 60, 60,   90, 90, 90, 90,
				120,120,120,120,  150,150,150,150,  180,180,180,180,  210,210,210,210,
				240,240,240,240,  270,270,270,270,  300,300,300,300,  330,330,330,330;
	.phase	list SPAM_ae[]	=   0;
	.phase		 SPAM_0		=   0;
	.phase		 SPAM_2		= 180;
	.phase	list SP_Ysta[]	= 270,270,270,270,  180,180,180,180,   90, 90, 90, 90,    0,  0,  0,  0;
	.phase	list SP_Conv[]	=   0,  0,180,180,  270,270, 90, 90,  180,180,  0,  0,   90, 90,270,270;
	.phase	list SP_Y90[]	=  90,270, 90,270,    0,180,  0,180,  270, 90,270, 90,  180,  0,180,  0;
	.phase	list SP_X90[]	=   0,  0,  0,  0,  270,270,270,270,  180,180,180,180,   90, 90, 90, 90;
	.phase	list SP_Ylst[]	=   0,  0,  0,  0,  270,270,270,270,  180,180,180,180,   90, 90, 90, 90;
	.phase	list SP_Xlst[]	=   0,  0,180,180,  270,270, 90, 90,  180,180,  0,  0,   90, 90,270,270;
	.long	extern list SPAM_r[]	= 0,2,0,2,  3,1,3,1,  2,0,2,0,  1,3,1,3;

!------------------------------------
! phases for 5/2 spin nuclei and 3Q experiment. Pathway: 0 => 3 => -1
!------------------------------------

	.phase	list I52_3QP1[]	=
				  0,  0,  0,  0,   60, 60, 60, 60,    0,  0,  0,  0,   60, 60, 60, 60,
				  0,  0,  0,  0,   60, 60, 60, 60,    0,  0,  0,  0,   60, 60, 60, 60,
				120,120,120,120,  180,180,180,180,  120,120,120,120,  180,180,180,180,
				120,120,120,120,  180,180,180,180,  120,120,120,120,  180,180,180,180,
				240,240,240,240,  300,300,300,300,  240,240,240,240,  300,300,300,300,
				240,240,240,240,  300,300,300,300,  240,240,240,240,  300,300,300,300;

	.phase	list I52_3QP2[]	=
				  0,  0,  0,  0,    0,  0,  0,  0,   90, 90, 90, 90,   90, 90, 90, 90,
				180,180,180,180,  180,180,180,180,  270,270,270,270,  270,270,270,270;

	.phase	list I52_Ysta[]	= 270,270,270,270, 90, 90, 90, 90;
	.phase	list I52_Conv[]	=   0,  0,180,180,  0,  0,180,180;
	.phase	list I52_Y90[]	=  90,270, 90,270, 90,270, 90,270;
	.phase	list I52_X90[]	=   0,  0,  0,  0,180,180,180,180;
	.phase	list I52_Ylst[]	=   0,  0,  0,  0,180,180,180,180;
	.phase	list I52_Xlst[]	=   0,  0,180,180,  0,  0,180,180;
! receiver phase :
	.long	extern list I52_r[]		= 0,2,0,2, 0,2,0,2;

!------------------------------------
! independent phases
!------------------------------------
 
	.phase		 Xdec		=	0;
	.phase		 Ydec		=	0;
	.phase		 Hdec		=	0;
	.phase	list phiAM[]	=  0,180;

	.phase	PHI1,PHI2,PHI3,phi_tmp,Ysta,Conv,Y90,X90,Xlst,Ylst;
 
	include "../includes/STANDARD_PARAMS";
	include "../includes/2D.inc";
	include "../includes/2Dutil2.inc";
	include "../includes/MQMASimp.inc";
	include "../includes/TPPM2.inc";
	include "../includes/NA_ERRORS.inc";
	
! ---------------------------------------------------
! Define error codes specific to this pulse program
! ---------------------------------------------------
define TAU_ERR 0x211
define TAU_ERROR_CODE USER_ERROR_BASE + TAU_ERR
comment "ERROR "TAU_ERROR_CODE "t1_evol, tau1 or tau2 is too short, or pulses are too long";

define TPPM_ERR 0x221
define TPPM_ERROR_CODE USER_ERROR_BASE + TPPM_ERR
comment "ERROR "TPPM_ERROR_CODE "leftover timing error increase tau1|2 by 0.1us";

!-----------------------------------------------------------------
!  Parameters are updated and reported to the acq panel by updates
!-----------------------------------------------------------------

USE_TPPM_PARAMS;

.update "rb=1.30*sw";
.update "aqtm=dw*(al/(Hyper_flag+1))";
! rd and ad are neglected when INEPT_flag=2;
.update "extm1=tau2*((INEPT_flag+1)/2)+pd+t1_evol";
.update "extm=(3.u+(2.*pw90Y)+2.*tau1+extm1+aqtm+(dw2*(al2-1)/2))";
.update "decT=.1u+(2.*tau1*(DEC_flag&1))+(aqtm*(DEC_flag&4)/4.)";
.update "decT2=.1u+(pw180X+pw180Y+(tau2*(DEC_flag&2)))*((INEPT_flag+1)/2)";
.update "txduty1=(pw180X+pw180Y+2.*pw90Y+pw90X+decT2+decT+(pw90Y*(INEPT_flag/2)))/extm";
.update "time1d=((na+dp)*extm*(Hyper_flag+1))/60.0";

!----------------------------------------------------
!  Executed once at start of experiment
!----------------------------------------------------

.program

! set dummy pulse counter
dpc = dp;
! some 2D initialisation stuff
dataxfer = al2;
CALC_HALFAL;
aqtm=dw*halfal;

CHECK_FAM_CONV;

/* calculate ratio for MQMAS */
if (Pnum==3) {
	phi_tmp=30.;
	if (SPAM_flag==1) {
		if (na%48==0) {
			if (Inum==2.5)
				MQratio=19./12.;
			else if (Inum==1.5)
				MQratio=7./9.;
			else
				error(SPIN_ERR);
		}
		else {
			error(NA048_ERR);
		}
	}
	else {
		if (na%96==0) {
			if (Inum==2.5)
				MQratio=19./12.;
			else if (Inum==1.5)
				MQratio=7./9.;
			else
				error(SPIN_ERR);
		}
		else {
			error(NA096_ERR);
		}
	}
}
else if (Pnum==5) {
	phi_tmp=18;
	error(SPIN_ERR);
}
else
	error(SPIN_ERR);
if (Pnum>(2.*Inum)) { error(SPIN_ERR); }

!!!! setting of proton channel delays assumes pw180=2*pw90
! this avoids decoupling during pulses sent on other channels
if (pw180X>pw180Y) {
	pw180H = pw180X;
}
else {
	pw180H = pw180Y;
}
if (pw90X>pw90Y) {
	pw90H = pw90X;
}
else {
	pw90H = pw90Y;
}

! TAUs calculations...
TAU1X = tau1 - (pw90X + pw180X)/2.;
TAU2X = tau2 - (pw90X + pw180X)/2.;
TAU1Y = tau1 - (pw90Y + pw180Y)/2.;
TAU2Y = tau2 - (pw90Y + pw180Y)/2.;
TAU1H = tau1 - (pw90H + pw180H)/2.;
TAU2H = tau2 - (pw90H + pw180H)/2.;
TAU4H = tau2 - (pw180H/2.);

!  initial delays
t1_out= t1_evol;
deltaXY=(pw90Y-pw90X)/2.;
deltaXH=(pw90Y-pw90H)/2.;

! TPPM phase init + cyclesPM calculations
CALC_TPPM_DEC(TAU1H,p2);
cyclesPM1=cyclesPM-1.;
leftOver1=TAU1H-cyclesPM1*tPMloop;

CALC_TPPM_CYCLES(TAU2H,cyclesPM2);
cyclesPM2=cyclesPM2-1.;
leftOver2=TAU2H-cyclesPM2*tPMloop;

CALC_TPPM_CYCLES(TAU2H,cyclesPM4);
cyclesPM4=cyclesPM4-1.;
leftOver4=TAU4H-cyclesPM4*tPMloop;

if (leftOver1 < 0.1u || leftOver2 < .1u || leftOver4 < 0.1u ) {
	error(TPPM_ERR);
}

CALC_TPPM_CYCLES(aqtm,cyclesPMaq);

if (INEPT_flag==1)
	rd = TAU2X + pw90Y/2. - ad;
if (INEPT_flag==2) {
	rd = TAU2X;
	ad = pw90Y;
	TAU4H = tau2 - (pw90Y + pw180H)/2.;
}

! actual txduty1 calculation
delays = t1_evol+pw1+pw2+t1_evol*MQratio+(2.*tau1)+aqtm+pd;
pulses = pw1+pw2+ZF_pulse+(pw90Y)+pw90X+pw180X+pw180Y;
if (INEPT_flag>0) {
	delays=delays+(2.*tau2);
	if (INEPT_flag==1) pulses=pulses+(pw180X+pw180Y);
	if (INEPT_flag==2) pulses=pulses+(pw180X+pw180Y)+pw90Y;
	if ( (DEC_flag&2) > 0) pulses=pulses+(2.*tau2);
}

if ((DEC_flag&1) > 0) pulses=pulses+(2.*tau1);
if ((DEC_flag&4) > 0) pulses=pulses+aqtm;

txduty1=(pulses/delays);
if (txduty1 > 0.2) {error(TXDUTY_ERR);}			!Duty factor too large

!check StartX and TAU3H not nul at start
TAU3H        = MQratio*(t1_evol)-.1u-deltaXH;

t1_evol = (1.1us + pw90Y/2. + ZF_pulse/2.)/MQratio;
!t1_out_2 = MQratio*t1_evol-.1u-pw90Y/2.-ZF_pulse/2.;
! na check
if ((na%48)>0) {error(NA048_ERR);}
! delays check
if (TAU1X<.3u || TAU2X<.3u || rd<.3u) {error(TAU_ERR);}
if (TAU1Y<.3u || TAU2Y<.3u ) {error(TAU_ERR);}
if (TAU1H<.3u || TAU2H<.3u || TAU3H<.3u || TAU4H<.3u) {error(TAU_ERR);}


! -------------------------------
! actual pulse prog. runtime loop
! -------------------------------

.start
loop2D{
	call start_phase_lists;
	call t1_out_calculations;
	loop1D
	{
		call select_PPG_phases;
		
		for (Hyper = 0, Hyper < (Hyper_flag+1), Hyper++) {
			if (Hyper == 0) { phi_mix = 0.; }
			else            { phi_mix = phi_tmp; }
			call mainseq;
			if (Hyper < Hyper_flag) { out   pd; }
		}
		scan  pd;               !end of seq. bookkeeping, recycle delay
	}
}

.end

!---------------------------------------
! START phase lists for SPAM or MQ
!---------------------------------------
start_phase_lists:
	if (SPAM_flag==1) {
		SPAM_e  	= SPAM_e.start;
		SPAM_r		= SPAM_r.start;
		SP_Ysta		= SP_Ysta.start;
		SP_Conv		= SP_Conv.start;
		SP_X90		= SP_X90.start;
		SP_Y90		= SP_Y90.start;
		SP_Ylst		= SP_Ylst.start;
		SP_Xlst		= SP_Xlst.start;
	}
	else {
		I52_3QP1	= I52_3QP1.start;
		I52_3QP2	= I52_3QP2.start;
		I52_r		= I52_r.start;
		I52_Ysta	= I52_Ysta.start;
		I52_Conv	= I52_Conv.start;
		I52_X90		= I52_X90.start;
		I52_Y90		= I52_Y90.start;
		I52_Ylst	= I52_Ylst.start;
		I52_Xlst	= I52_Xlst.start;
	}
return;

!---------------------------------------
! SELECT phase lists for SPAM or MQ
!---------------------------------------
select_PPG_phases:
	if (SPAM_flag==1) {
		PHI1 = @SPAM_e++;
		PHI2 = SPAM_0;
		PHI3 = SPAM_0;
		Ysta = @SP_Ysta++;
		Conv = @SP_Conv++;
		X90  = @SP_X90++;
		Y90	 = @SP_Y90++;
		Xlst = @SP_Xlst++;
		Ylst = @SP_Ylst++;
		aqph = @SPAM_r++; 
	}
	else {
		PHI1 = @I52_3QP1++;
		PHI2 = @I52_3QP2++;
		Ysta = @I52_Ysta++;
		Conv = @I52_Conv++;
		X90  = @I52_X90++;
		Y90	 = @I52_Y90++;
		Xlst = @I52_Xlst++;
		Ylst = @I52_Ylst++;
		aqph = @I52_r++; 
	}
return;

!---------------------------------------
! INCREASE t1 evolution time
!---------------------------------------
t1_out_calculations:
	t1_out_1 = t1_evol+(dw2*index2);
	t1_out_2 = MQratio*(t1_evol+(dw2*index2))-.1u;
	StartX	 = t1_out_2 + deltaXY;
	TAU3H	 = t1_out_2 + deltaXH;
return;

!---------------------------------------
! MAIN pulse program
!---------------------------------------
mainseq:

	@PMphase=@PMphase.start;	! reset TPPM phase list

!S spins, channel X, acq

Async;
!!   channel 1     observed channel     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
ch++ ch1;
		out   time(3u)	ch1: SC(scX);
		out   time(1u)	ch1: A(aX);
! SPAM part : 
! repeating delays of ch2 to adjust timing before INEPT part
		out   pw1;						! excitation MQ level
		out   time(t1_out_1);
		out   timeFAM;					! reconversion with FAM
if (SPAM_flag==1) {
		out   time(.1u);
		out   time(1u);
		out   time(.4u);
		out   ZF_pulse;
}
else {
		out   time(.1u);
		out   time(.4u);
}

		out   time(StartX);
! INEPT part
		out   pw90X;
		out   TAU1X		ch1: P(Conv);
		out   pw180X	ch1: TG;
		out   TAU1X		ch1: P(X90);
		out   pw90X		ch1: TG;
	if (INEPT_flag>0) {														!INEPT ref. and INEPT+
		out   TAU2X		ch1: P(Xlst);
		out   pw180X	ch1: TG;
	}
		out   rd		ch1: TB | P(p);
		out   ad		ch1: RE | TB;
		outAQ   dw		ch1: RE | TB, halfal;
		out   time(1m) ch1: TB; ! safe resync ???
Sync;
ch--;


!!   channel 2  (Y or I) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
ch++ ch2;

		out   time(3u)			ch2: SC(scMQ);
		out   time(1u)			ch2: AP(aMQ,PHI1+phi_mix);
! MQ part
! MQ pulses on Y
        out   pw1               ch2: TG;        ! excitation MQ level
        out   time(t1_out_1)    ch2: AP(aMQ,PHI2);
 
        OUT_CONVERSION_PULSE_CH(ch2); ! pulse(s) FAM depending on FAM_flag
 
if (SPAM_flag==1) {
        out   time(.1u);
        out   time(1u)			ch2: SC(scY);
        out   time(.4u)			ch2: AP(aY,PHI3);
        out   ZF_pulse			ch2: TG;
        out   time(t1_out_2)	ch2: P(Ysta);
}
else {
        out   time(.1u);
        out   time(.3u)			ch2: SC(scY);
        out   time(t1_out_2)	ch2: AP(aY,Ysta);
}

! INEPT part
		out   pw90Y			ch2: TG;

		out   TAU1Y			ch2: P(Conv);
		out   pw180Y		ch2: TG;
		out   TAU1Y			ch2: P(Y90);
		out   pw90Y			ch2: TG;
	if (INEPT_flag>0) {								! INEPTR and INEPT+
		out   TAU2Y			ch2: P(Conv);
		out   pw180Y		ch2: TG;
		out   TAU2Y			ch2: P(Ysta);
	}
	if (INEPT_flag==2) {							! INEPT+
		out   pw90Y			ch2: TG;
	}
	out   aqtm		ch2: TB;	!INEPT or INEPTR

waitS;
ch--;

!!   channel 3 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
ch++ ch3;
		out   time(3u)		ch3: SC(scH);
		out   time(2u)		ch3: AP(aHdec,Hdec);
!if (Dec during MQMAS t1)        {try to avoid pulses}
! repeating delays on ch1 to adjust timing before INEPT part
	out   pw1;						! excitation MQ level
	out   time(t1_out_1);
	out   timeFAM;					! reconversion with FAM

if (SPAM_flag==1) {
		out   time(.1u);
		out   time(1u);
		out   time(.4u);
		out   ZF_pulse;
}
else {
		out   time(.1u);
		out   time(.4u);
}
	out   time(TAU3H)	;

!!!!!!!!!!!!!! decoupling during tau1 !!!!!!!!!!!!!!!!!!!!!!
	out   pw90H			;

	if ((DEC_flag&1)>0) {				
if (Dmode_flag==0) { 					! CW decoupling
	out   TAU1H			ch3: TG;
}
else {
	RUN_TPPM_DEC(ch3,cyclesPM1);
	out leftOver1;
}
	out   pw180H	;
if (Dmode_flag==0) { 					! CW decoupling
	out   TAU1H				ch3: TG;
}
else {
	RUN_TPPM_DEC(ch3,cyclesPM1);
	out leftOver1;
}
		out   pw90H	;
	}
	else {								! no decoupling
		out   TAU1H			;
		out   pw180H		;
		out   TAU1H			;
		out   pw90H			;
	}

	if (INEPT_flag>0) {					! INEPTR and INEPT+

!!!!!!!!!!!!!! decoupling during tau2 !!!!!!!!!!!!!!!!!!!!!!

			if((DEC_flag&2)>0) {	
if (Dmode_flag==0) { 				! CW decoupling
	out   TAU2H			ch3: TG;
}
else {
	RUN_TPPM_DEC(ch3,cyclesPM2);
	out leftOver2;
}
		out   pw180H	;
if (Dmode_flag==0) { 				! CW decoupling
	out   TAU4H			ch3: TG;
}
else {
	RUN_TPPM_DEC(ch3,cyclesPM4);
	out leftOver4;
}
			}
			else {					! no decoupling
	out   TAU2H			;
	out   pw180H		;
	out   TAU4H			;
			}
	}

	if (INEPT_flag==2)							! INEPT+
		out  pw90Y		;

!!!!!!!!!!!!!!!!!!! decoupling during acquisition !!!!!!!!!!!!!1

	if((DEC_flag&4)>0) {
if (Dmode_flag==0) { 				! CW decoupling
		out   aqtm		ch3: TG;
}
else {
		RUN_TPPM_DEC(ch3,cyclesPMaq);
}
	}
	else	{						! no decoupling
		out   aqtm		ch3: TB;
	}

! security blanking transmitter
	out   time(.1u)					ch3: TB;

waitS;
ch--;

return;

