! ---------------------------------------------------
! Define error codes specific to CP errors
! ---------------------------------------------------

define RAMP_ERR 0x800
define RAMP_ERROR_CODE USER_ERROR_BASE + RAMP_ERR
comment "ERROR "RAMP_ERROR_CODE "ramp amplitude will go negative";

! ---------------------------------------------------
! variables required for ramp CP (vacp)
! ---------------------------------------------------
DEFINE USE_RAMP_PARAMS
	.time	autofix	CT					= .1m;
	.scale	extern	scXcp "X CP scale"	= 0.001;
	.scale	extern	scYcp "X CP scale"	= 0.001;
	.scale	extern	scHcp "X CP scale"	= 0.001;
	.ampl	extern	step_size			= .01;
	.ampl	aHCP,aXCP,aYCP,aHstep,aXstep,aYstep;
	.long	extern	RAMP_flag			= 1;
	.long	extern	DD_flag				= 0;	! decoupling flag
	.float	extern	no_steps			= 11.;
ENDDEF

! ---------------------------------------------------
! calculate CP ramp variables
! ---------------------------------------------------
DEFINE CALC_RAMP
	CT = (ct/no_steps);
	aHstep = 0.0;
	aXstep = 0.0;
	aYstep = 0.0;
	if (RAMP_flag==1) { aHstep = step_size; }
	if (RAMP_flag==2) { aXstep = step_size; }
	aHCP = (aHcp - (aHstep*no_steps));
	aXCP = (aXcp - (aXstep*no_steps));
	if ( aHCP<0.0 || aXCP<0.0 ) { error(RAMP_ERR); }		!Negative amplitude not allowed.
ENDDEF

DEFINE CALC_RAMP_STEP(CPINI,CPAMPL,CPSTEP)
	CT = (ct/no_steps);
	aHstep = 0.0;
	aXstep = 0.0;
	aYstep = 0.0;
	CPSTEP = step_size;
	CPAMPL = (CPINI - (CPSTEP*no_steps));
	if ( CPAMPL<0.0 ) { error(RAMP_ERR); }		!Negative amplitude not allowed. 
	!aHCP = (aHcp - (aHstep*no_steps));
	!aXCP = (aXcp - (aXstep*no_steps));
	!aYCP = (aYcp - (aYstep*no_steps));
	!if ( aHCP<0.0 || aXCP<0.0 || aYCP<0.0 ) { error(RAMP_ERR); }		!Negative amplitude not allowed.
ENDDEF

! -------------------------------------------
! use in pulse prog. runtime loop
! -------------------------------------------
! -------------------------------------------
! use in pulse prog. runtime loop
! -------------------------------------------

DEFINE INIT_RAMP
	aHCP = aHcp;
	aXCP = aXcp;
	aYCP = aYcp;
ENDDEF

DEFINE RUN_RAMP_CP(Xtmp,Htmp)
do(no_steps){
	out	 CT		ch1: TG | AP(aXCP,Xtmp)		ch2: TG | AP(aHCP,Htmp); 	!output CP pulse
	aHCP = aHCP - aHstep;
	aXCP = aXCP - aXstep;
	}
ENDDEF

DEFINE RUN_RAMP_HXCP(Xtmp,XCHN,Htmp,HCHN)
	out  time(.12u)	XCHN:P(Xtmp)	HCHN:P(Htmp);
do(no_steps){
	out   CT		XCHN:TG|A(aXCP)		HCHN:TG|A(aHCP); 	!output CP pulse
	aHCP = aHCP - aHstep;
	aXCP = aXCP - aXstep;
	}
ENDDEF

DEFINE RUN_RAMP_XYCP(Xtmp,XCHN,Ytmp,YCHN)
	out  time(.2us) XCHN: P(Xtmp)		YCHN: P(Ytmp);
do(no_steps){
	out	 CT			XCHN: TG|A(aXCP)	YCHN: TG|A(aYCP); 	!output CP pulse
	aYCP = aYCP - aYstep;
	aXCP = aXCP - aXstep;
	}
ENDDEF

