!-----------------------------------------------------------------
! MQMAS.inc
! for MQMAS programs
!-----------------------------------------------------------------
!	comments to:	Jerzy W. Wiench
!		email:		jwiench@iastate.edu
!		Address:	Iowa State University,
!					Ames Laboratory,
!					229 Spedding Hall
!					Ames, IA 50011, USA
!-----------------------------------------------------------------

!	references:
!	1.	MQMAS: A.Medek, J.S.Harwood, L.Frydman, J.Am.Chem.Soc., 117 (1995) 12779.
!	2.	Z-filter: J.P.Amoureux, C.Fernandez, S.Steuernagel, J.Magn.Reson. A123 (1996) 116.
!	3.	FAM-I  (3/2): P.K.Madhu, A.Goldbourt, L.Frydman, S.Vega, Chem.Phys.Lett., 307 (1999) 41.
!	4.	FAM-I  (3/2): P.K.Madhu, A.Goldbourt, L.Frydman, S.Vega, J.Chem.Phys., 112 (2000) 2377.
!	5.	FAM-II (5/2): A.Goldbourt, P.K.Madhu, S.Vega, Chem.Phys.Lett., 320 (2000) 448.
!	6.	TPPM: A.E.Bennett, C.M.Rienstra, M.Auger, K.V.Lakshmi, R.G.Griffin,
!		J.Chem.Phys., 103 (1995) 6951.

!		Phase_flag	select phases for pathways:
!					0	both:	0  ->  +/-3  ->  0  ->  -1  (acq) - na multiple of 24 or 40
!					1	top:	0  ->   +3   ->  0  ->  -1  (acq) - na multiple of 32 or 48
!					2	bottom:	0  ->   -3   ->  0  ->  -1  (acq) - na multiple of 32 or 48
!		Hdec_flag	select H decoupling mode:
!					0	coupled spectrum
!					1	CW decoupling; use proper aHdec and scH
!					2	TPPM decoupling; use proper aHdec, scH, pw_dec=pw180H at
!						aHdec amplitude, p2=15-30 (decoupler phase shift see ref. [6])
!		Inum		provide spin number
!		Pnum		provide p-quantum number. It selects proper phase cycling for 3Q or 5Q exp.

! ---------------------------------------------
! pulses, delays and flags
! ---------------------------------------------
	.time	extern		autofix	pw1 		= 3.1us;
	.time	extern		autofix pw2			= 0.6us;
	.time	extern		autofix ZF_delay	= 1.0ms;
	.time	extern		autofix ZF_pulse	= 1.0us;
	.long	extern		FAM_flag			= 0;
	.time	extern		autofix	timeFAM		= 1us;
	.long	extern		Pnum				= 3;
	.float	extern		Inum				= 1.5;

! ---------------------------------------------
! FAM pulses, delays and counters
! ---------------------------------------------
	.time	extern		autofix pw1AM		= 0.6us;
	.time	extern		autofix t1AM	 	= 0.3us;
	.time	extern				timeP		= 1us;
	.time	extern				timeD		= 1us;
	.long	extern		n1AM				= 1;

	.time	extern		autofix pw2AM		= 0.6us;
	.time	extern		autofix t2AM	 	= 0.3us;
	.long	extern		n2AM				= 1;
	.long	x,n2AM2;

! ---------------------------------------------
! aplitudes and scalars
! ---------------------------------------------
	.scale  extern				scMQ_ZF		= 0.1;
	.ampl	extern				aMQ_ZF		= 0.0;
	.scale  extern				scMQ		= 0.1;
	.ampl	extern				aMQ			= 0.0;
	.scale  extern				scMQ_1		= 0.1;
	.ampl	extern				aMQ_1		= 0.0;
	.scale  extern				scMQ_2		= 0.1;
	.ampl	extern				aMQ_2		= 0.0;

! ---------------------------------------------------
! Define error codes specific to this pulse program
! ---------------------------------------------------
define SPIN_ERR 0x401
define SPIN_ERROR_CODE USER_ERROR_BASE + SPIN_ERR
comment "ERROR "SPIN_ERROR_CODE "Select Inum = 1.5, 2.5, 3.5, or 4.5; Pnum = 3 or 5";

define T1EVOL_ERR 0x411
define T1EVOL_ERROR_CODE USER_ERROR_BASE + T1EVOL_ERR
comment "ERROR "T1EVOL_ERROR_CODE "t1_evol is too short!";

!++++++++++++++++++++++++++++++++++++
!	Pulse cycles
!++++++++++++++++++++++++++++++++++++

DEFINE ZF_PAHSE_BOTH
!-------------------------------------------------------------------------------------------
! 6 phases selecting both pathways of 3Q experiment 0 => +/-3 => 0 => -1
!-------------------------------------------------------------------------------------------
    .phase	list Both3Q_1[]	= 0, 60,120,180,240,300; 
	.phase 	list Both3Q_2[]	= 0,  0,  0,  0,  0,  0;
	.phase	list Both3Q_3[]	= 0,  0,  0,  0,  0,  0,  90, 90, 90, 90, 90, 90,
							180,180,180,180,180,180, 270,270,270,270,270,270;
	.long	extern list Both3Q_r[]	= 0,2,0,2,0,2, 1,3,1,3,1,3,
							 		  2,0,2,0,2,0, 3,1,3,1,3,1;
!-------------------------------------------------------------------------------------------
! 10 phases selecting both pathways of 5Q experiment 0 => +/-5 => 0 => -1
!-------------------------------------------------------------------------------------------
    .phase	list Both5Q_1[]	 =  0,  0;
    .phase	list Both5Q_2[]  =  0, 36, 72,108,144,180,216,252,288,324;  
	.phase	list Both5Q_3[]	 =  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
							   90, 90, 90, 90, 90, 90, 90, 90, 90, 90,
							  180,180,180,180,180,180,180,180,180,180,
							  270,270,270,270,270,270,270,270,270,270; 
    .long extern list Both5Q_r[] =  0,2,0,2,0,2,0,2,0,2, 1,3,1,3,1,3,1,3,1,3,
									2,0,2,0,2,0,2,0,2,0, 3,1,3,1,3,1,3,1,3,1;
ENDDEF

DEFINE ZF_PHASE_ONE
!-------------------------------------------------------------------------------------------
! 8 phases selecting echo or anti-echo pathway of 3Q experiment
! phases for pulses pw1, pw2, and ZF_pulse are the same
! receiver phases are different!
!-------------------------------------------------------------------------------------------
	.phase	list One3Q_1[]	= 0, 90,180,270;
	.phase	list One3Q_2[]	= 0,  0,  0,  0,   0,  0,  0,  0,   0,  0,  0,  0,   0,  0,  0,  0,
							315,315,315,315, 315,315,315,315, 315,315,315,315, 315,315,315,315;
    .phase 	list One3Q_3[]	= 0,  0,  0,  0,  90, 90, 90, 90, 180,180,180,180, 270,270,270,270,
							315,315,315,315,  45, 45, 45, 45, 135,135,135,135, 225,225,225,225;

!-------------------------------------------------------------------------------------------
! receiver phases for echo pathway of 3Q experiment ("bottom one") 0 => -3 => 0 => -1 acq
!-------------------------------------------------------------------------------------------
    .long 	extern list BEcho3Q_r[]	= 0,3,2,1, 1,0,3,2, 2,1,0,3, 3,2,1,0,
									  1,0,3,2, 2,1,0,3, 3,2,1,0, 0,3,2,1;

!-------------------------------------------------------------------------------------------
! receiver phases for anti-echo pathway of 3Q experiment ("top one") 0 => +3 => 0 => -1 acq
!-------------------------------------------------------------------------------------------
    .long 	extern list TEcho3Q_r[]	= 0,1,2,3, 1,2,3,0, 2,3,0,1, 3,0,1,2,
									  2,3,0,1, 3,0,1,2, 0,1,2,3, 1,2,3,0;

!-------------------------------------------------------------------------------------------
! 12 phases selecting one pathway of 5Q experiment 0 => +5 (or -5)  => 0 => -1
!-------------------------------------------------------------------------------------------
	.phase	list One5Q_1[]	=  0, 90,180,270, 240,330, 60,150, 120,210,300, 30;
	.phase	list One5Q_2[]	=  0,  0,  0,  0,  30, 30, 30, 30,  60, 60, 60, 60;
	.phase	list One5Q_3[]	=  0,  0,  0,  0,  30, 30, 30, 30,  60, 60, 60, 60,
							  90, 90, 90, 90, 120,120,120,120, 150,150,150,150,
							 180,180,180,180, 210,210,210,210, 240,240,240,240,
							 270,270,270,270, 300,300,300,300, 330,330,330,330;
	.long	extern list One5Q_r[]	= 0,1,2,3;
ENDDEF

DEFINE SPAM_PHASE_ALL
!-------------------------------------------------------------------------------------------
! 12 phases selecting 3Q echo and antiecho pathways for all spins
!-------------------------------------------------------------------------------------------
	.phase	list SPAM_e[]			=   0, 30, 60, 90, 120,150,180,210, 240,270,300,330;
	.phase	list SPAM_ae[]			=  60, 30,  0, 90, 180,150,120,210, 300,270,240,330;
	.phase		 SPAM_0				=   0;					! no cycling on pulse 2
	.phase		 SPAM_2				= 180;					! no cycling on pulse 3
	.long	extern list SPAM_r[]	= 0,3,2,1;
ENDDEF 

DEFINE SPAM_MQ_PHASE_ALL
!-------------------------------------------------------------------------------------------
! 12 phases selecting 3Q echo and antiecho pathways for all spins
! 20 phases selecting 5Q echo and antiecho pathways for all spins
!-------------------------------------------------------------------------------------------
	.phase	list SPAM_3Q_e[]		=   0, 30, 60, 90, 120,150,180,210, 240,270,300,330;
	.phase	list SPAM_3Q_ae[]		=  60, 30,  0, 90, 180,150,120,210, 300,270,240,330;
	.phase	list SPAM_5Q_ae[]		=   0, 18, 36, 54,  72, 90,108,126, 144,162,180,198, 216,234,252,270, 288,306,324,342;
	.phase	list SPAM_5Q_e[]		=  36, 18,  0, 54, 108, 90, 72,126, 180,162,144,198, 252,234,216,270, 324,306,288,342;
	.phase		 SPAM_0				=   0;					! no cycling on pulse 2
	.phase		 SPAM_2				= 180;					! no cycling on pulse 3
	.long	extern list SPAM_r[] = 0,3,2,1;
ENDDEF

!++++++++++++++++++++++++++++++++++++
!	end phase section
!++++++++++++++++++++++++++++++++++++

! =========================================================
!  PROGRAM SECTION (executed once at start of run)
! =========================================================


DEFINE CHECK_FAM_CONV
	n2AM2 = 2 * n2AM;
	if (FAM_flag==0) { 
		timeP = timeP + pw2;
       		timeFAM=pw2+t2AM;
	}
	if (FAM_flag==1) { 
		timeP = timeP + pw2 + pw2AM; 
		timeD = timeD + (2. * t2AM); 
		timeFAM= pw2 + pw2AM + (2. * t2AM);
	}
	if (FAM_flag==2) { 
		timeP = timeP + 2.*(pw2 + pw2AM); 
		timeD = timeD + (4.*t2AM);
       		timeFAM= 2.*(pw2 + pw2AM) + (4.*t2AM);
	}
	if (FAM_flag==3) { 
		timeP = timeP + (n2AM2*pw2);
		timeD = timeD + (n2AM2*t2AM);
		timeFAM=(n2AM2*pw2)+(n2AM2*t2AM);
	}
ENDDEF

! -------------------------------
! actual pulse prog. runtime loop
! -------------------------------

DEFINE START_PHASE_BOTH
if (Pnum==3) {
	Both3Q_1	= Both3Q_1.start;
	Both3Q_2	= Both3Q_2.start;
	Both3Q_3	= Both3Q_3.start;
	Both3Q_r	= Both3Q_r.start;
}
if (Pnum==5) {
	Both5Q_1	= Both5Q_1.start;
	Both5Q_2	= Both5Q_2.start;
	Both5Q_3	= Both5Q_3.start;
	Both5Q_r	= Both5Q_r.start;
}
ENDDEF

DEFINE START_PHASE_ONE
	One3Q_1 = One3Q_1.start;
	One3Q_2 = One3Q_2.start;
	One3Q_3 = One3Q_3.start;
    TEcho3Q_r	= TEcho3Q_r.start;
    BEcho3Q_r	= BEcho3Q_r.start;
ENDDEF

DEFINE START_PHASE_SPAM
	SPAM_e  = SPAM_e.start;
	SPAM_ae = SPAM_ae.start;
	SPAM_r  = SPAM_r.start;
ENDDEF

DEFINE START_PHASE_SPAM_MQ
	SPAM_3Q_e  = SPAM_3Q_e.start;
	SPAM_3Q_ae = SPAM_3Q_ae.start;
	SPAM_5Q_e  = SPAM_5Q_e.start;
	SPAM_5Q_ae = SPAM_5Q_ae.start;
	SPAM_r     = SPAM_r.start;
ENDDEF

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
! phase selection lines to chose an apropiate pathway
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE SELECT_PHASE_BOTH
	!----------------------------------------------------------------------------------
	! 6 phases selecting both pathways for 3Q(5Q) experiment: 0 => +/-3(5) => 0 => -1
	!----------------------------------------------------------------------------------
	if (Pnum==3) {
		PHI1=@Both3Q_1++;
		PHI2=@Both3Q_2++;
		PHI3=@Both3Q_3++;
		aqph=@Both3Q_r++;
	}
	else {
		PHI1=@Both5Q_1++;
		PHI2=@Both5Q_2++;
		PHI3=@Both5Q_3++;
		aqph=@Both5Q_r++;
	}
ENDDEF

DEFINE SELECT_PHASE_ONE
	if (Pnum==3) {
	!----------------------------------------------------------------------------------
	! 8 phases selecting echo or anti-echo pathway
	! phases for pulses pw1, pw2, and ZF_pulses are the same
	! receiver phases are different!
	!----------------------------------------------------------------------------------
		PHI1=@One3Q_1++;
		PHI2=@One3Q_2++;
		PHI3=@One3Q_3++;

	!----------------------------------------------------------------------------------
	! receiver phaseS for anti-echo pathway ("top one") 0 => +3 => 0 => -1 acq
	!----------------------------------------------------------------------------------
		if (Phase_flag == 1) { aqph=@TEcho3Q_r++; }

	!----------------------------------------------------------------------------------
	! receiver phases for echo pathway ("bottom one") 0 => -3 => 0 => -1 acq
	!----------------------------------------------------------------------------------
		else { aqph=@BEcho3Q_r++; }
	}
	else {error(SPIN_ERR);}
ENDDEF

DEFINE SELECT_PHASE_SPAM
	!----------------------------------------------------------------------------------
	! the same second pulse phase and receiver phase
	!----------------------------------------------------------------------------------
	PHI2=SPAM_0;
	aqph=@SPAM_r++; 
ENDDEF

! Actually here ECHO means 0 => +3 => 0 => -1 pathway
DEFINE SELECT_PHASE_SPAM_ECHO
	PHI3=SPAM_0;
	PHI1=@SPAM_e++;
ENDDEF

! Actually here AECHO means 0 => -3 => 0 => -1 pathway
DEFINE SELECT_PHASE_SPAM_AECHO
	PHI3=SPAM_2;
	PHI1=@SPAM_ae++;
ENDDEF

! Actually here ECHO means 0 => +3 => 0 => -1 pathway
DEFINE SELECT_PHASE_SPAM_MQ_ECHO
	if (Pnum==3) {
		if (Inum>2.1) {
			PHI1=@SPAM_3Q_e++;
			PHI3=SPAM_0;
		}
		else {
			PHI1=@SPAM_3Q_ae++;
			PHI3=SPAM_2;
		}
	}
	else {
		PHI1=@SPAM_5Q_e++;
		PHI3=SPAM_2;
	}
ENDDEF

! Actually here AECHO means 0 => -3 => 0 => -1 pathway
DEFINE SELECT_PHASE_SPAM_MQ_AECHO
	if (Pnum==3) {
		if (Inum>2.1) {
			PHI1=@SPAM_3Q_ae++;
			PHI3=SPAM_2;
		}
		else {
			PHI1=@SPAM_3Q_e++;
			PHI3=SPAM_0;
		}
	}
	else {
		PHI1=@SPAM_5Q_ae++;
		PHI3=SPAM_0;
	}
ENDDEF

!++++++++++++++++++++++++++++++++
! end phase selection section
!++++++++++++++++++++++++++++++++

DEFINE OUT_CONVERSION_PULSE
	if (FAM_flag == 0) {
	out   	pw2					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	}

	else if (FAM_flag == 1) {
	out   	pw2					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	out   	pw2AM				ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	}

	else if (FAM_flag == 2) {
	out   	pw2					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	out   	pw2					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	out	pw2AM					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	out	pw2AM					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
	}

	else {
		for (x = 0, x < n2AM2, x++) {
	out   	pw2					ch1: TG;
	out		t2AM				ch1: P(PHI2+@phiAM++);
		}
	}
ENDDEF

DEFINE OUT_CONVERSION_PULSE_CH( uch )
	if (FAM_flag == 0) {
	out   	pw2					uch: TG;
	!out		t2AM				uch: P(PHI2+@phiAM++);
	}

	else if (FAM_flag == 1) {
	out   	pw2					uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	out   	pw2AM				uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	}

	else if (FAM_flag == 2) {
	out   	pw2					uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	out   	pw2					uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	out	pw2AM				uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	out	pw2AM				uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
	}

	else {
		for (x = 0, x < n2AM2, x++) {
	out   	pw2					uch: TG;
	out		t2AM				uch: P(PHI2+@phiAM++);
		}
	}
ENDDEF
